@@ -97,6 +97,7 @@ class TtuHook(BaseHook, LoggingMixin):
                 self.log.debug(bteq_file)
                 tmpfile.write(bytes(bteq_file,'UTF8'))
                 tmpfile.flush()
+                tmpfile.seek(0)
 
                 conn['sp'] = subprocess.Popen(['bteq'],
                     stdin=tmpfile,
@@ -124,7 +125,7 @@ class TtuHook(BaseHook, LoggingMixin):
                 if xcom_push_flag:
                     return line
 
-    def execute_tdload(self, input_file, table, delimiter=';', working_database=None, encoding='UTF8', xcom_push_flag=False, raise_on_rows_error=False, raise_on_rows_duplicated=False):
+    def execute_tdload(self, input_file, table, delimiter=';', working_database=None, encoding='UTF8', xcom_push_flag=False, raise_on_rows_error=False, raise_on_rows_duplicated=False, debug=False, restart_limit=0):
         """
         Load a CSV file to Teradata Table (previously created) using tdload binary.
         Note: You need to strip header of the CSV. tdload only accepts rows, not header.
@@ -160,7 +161,9 @@ class TtuHook(BaseHook, LoggingMixin):
                             fload_out_path,
                             fload_checkpoint_path,
                             conn['ttu_max_sessions'],
-                            working_database
+                            working_database,
+                            debug,
+                            restart_limit
                          ),
             stdout=subprocess.PIPE, stderr=subprocess.STDOUT,preexec_fn=os.setsid)
         line = ''
@@ -170,7 +173,7 @@ class TtuHook(BaseHook, LoggingMixin):
         rows_error=0
         rows_duplicated=0
         for line in iter(conn['sp'].stdout.readline, b''):
-            #line = line.decode(conn['console_output_encoding']).strip()
+            line = line.decode(conn['console_output_encoding']).strip()
             self.log.info(line)
             if rows_in_error_tables_line in line:
                 # check if we have error rows
@@ -200,7 +203,7 @@ class TtuHook(BaseHook, LoggingMixin):
             return line
 
 
-    def execute_tptexport(self, sql, output_file, delimiter = ';', encoding='UTF8', xcom_push_flag=False, double_quote_varchar=True):
+    def execute_tptexport(self, sql, output_file, delimiter = ';', encoding='UTF8', spool_mode='SPOOL', xcom_push_flag=False, double_quote_varchar=True):
         """
         Export a table from Teradata Table using tpt binary.
         Note: The exported CSV file does not contains header row
@@ -209,6 +212,7 @@ class TtuHook(BaseHook, LoggingMixin):
         :param encoding : encoding of the file to load
         :param table : output table
         :param max_sessions : max sessions to use
+        :param spool_mode : ref https://docs.teradata.com/reader/tRbhWsU75TDpkqzyEZReyA/2gbczYmS~PRXRVKD0Dngtg
         :param xcom_push_flag: flag for pushing last line of BTEQ Log to XCom
         :param double_quote_varchar: if true, replace quotes with escaping char for Teradata SQL in TPT
         """
@@ -227,12 +231,13 @@ class TtuHook(BaseHook, LoggingMixin):
         self.log.info("""Exporting SQL '""" + sql + """' to file """ + output_file + """ using TPT Export""")
 
         with TemporaryDirectory(prefix='airflowtmp_ttu_tpt') as tmp_dir:
-            with NamedTemporaryFile(dir=tmp_dir, prefix=str(uuid.uuid4()), mode='wb') as f:
+            with NamedTemporaryFile(dir=tmp_dir, prefix=uuid.uuid4().hex, mode='wb') as f:
                 f.write(bytes(self._prepare_tpt_export_script(
                                         sql,
                                         output_file,
                                         encoding,
                                         delimiter,
+                                        spool_mode,
                                         conn['host'],
                                         conn['login'],
                                         conn['password'],
@@ -244,14 +249,14 @@ class TtuHook(BaseHook, LoggingMixin):
                              "location :{0}".format(fname))
                 f.seek(0)
                 conn['sp'] = subprocess.Popen(
-                    ['tbuild', '-f', fname, 'airflow' + '_tpt_'  + str(uuid.uuid4())],
+                    ['tbuild', '-f', fname, 'airflow' + '_tpt_'  + uuid.uuid4().hex],
                     stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                     preexec_fn=os.setsid)
 
                 line = ''
                 error_line = 'unknown reasons. Please see full TPT Output for more details.'
                 for line in iter(conn['sp'].stdout.readline, b''):
-                    #line = line.decode(conn['console_output_encoding']).strip()
+                    line = line.decode(conn['console_output_encoding']).strip()
                     self.log.info(line)
                     if "error" in line:
                         #Just save the last error
@@ -294,7 +299,7 @@ class TtuHook(BaseHook, LoggingMixin):
         return "\n".join(bteq_list)
 
     @staticmethod
-    def _prepare_tdload_command(input_file, host, login, password, encoding, table, delimiter, log_path, checkpoint_path, max_sessions, working_database, job_name= 'airflow_tdload') -> str:
+    def _prepare_tdload_command(input_file, host, login, password, encoding, table, delimiter, log_path, checkpoint_path, max_sessions, working_database, debug, restart_limit, job_name= 'airflow_tdload') -> str:
         """
         Prepare a tdload file with connection parameters for loading data from file
         :param input_file : bteq sentences to execute
@@ -320,20 +325,24 @@ class TtuHook(BaseHook, LoggingMixin):
         tdload_command += ['-d'] + [delimiter]
         tdload_command += ['-L'] + [log_path]
         tdload_command += ['-r'] + [checkpoint_path]
+        tdload_command += ['-R'] + [str(restart_limit)]
         tdload_command += ['--TargetMaxSessions'] + [str(max_sessions)]
         if working_database:
             tdload_command += ['--TargetWorkingDatabase'] + [working_database]
-        tdload_command += [ "%s_%s" % (job_name, uuid.uuid1()) ] #Job Name
+        if debug:
+            tdload_command += ['-x']
+        tdload_command += [ "%s_%s" % (job_name, uuid.uuid4().hex) ] #Job Name
         return tdload_command
 
     @staticmethod
-    def _prepare_tpt_export_script(sql, output_file, encoding, delimiter, host, login, password, max_sessions, job_name= 'airflow_tptexport') -> str:
+    def _prepare_tpt_export_script(sql, output_file, encoding, delimiter, spool_mode, host, login, password, max_sessions, job_name= 'airflow_tptexport') -> str:
         """
         Prepare a tpt script file with connection parameters for exporting data to CSV
         :param sql : SQL sentence to export
         :param output_file : path to output file
-        :encoding : encoding of exported CSV file (see teradata docs for possible value)
-        :delimiter : Delimiter for exported CSV file
+        :param encoding : encoding of exported CSV file (see teradata docs for possible value)
+        :param delimiter : Delimiter for exported CSV file
+        :param spool_mode : ref https://docs.teradata.com/reader/tRbhWsU75TDpkqzyEZReyA/2gbczYmS~PRXRVKD0Dngtg
         :param host : Teradata Host
         :param login : username for login
         :param password : password for login
@@ -368,11 +377,12 @@ class TtuHook(BaseHook, LoggingMixin):
                                     UserPassword = '{password}',
                                     SelectStmt = '{sql}',
                                     TdpId = '{host}',
-                        MaxSessions = {max_sessions}
+                                    MaxSessions = {max_sessions},
+                                    SpoolMode = '{spool_mode}'
                             )
                     );
             );
             '''.format(filename=output_file, encoding=encoding, delimiter=delimiter, username=login,
-                       password=password, sql=sql, host=host, max_sessions=max_sessions, job_name = job_name)
+                       password=password, sql=sql, host=host, max_sessions=max_sessions, job_name = job_name, spool_mode=spool_mode)
 
 
